SystemOrganization addCategory: #'Ma Statistics-Info'!
SystemOrganization addCategory: #'Ma Statistics-Domain'!

MaObject subclass: #MaStat
	instanceVariableNames: 'last min max count sum'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma Statistics-Domain'!

!MaStat commentStamp: 'cmm 5/6/2008 12:31' prior: 0!
The name, "MaStat" is purposefully abbreviated because its capability, itself, is abbreviated.  It is not intended to ever develop into a "statistics framework", only to serve basic statistical purposes.!

MaStat subclass: #MaHistoricalStat
	instanceVariableNames: 'name startTime endTime'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma Statistics-Domain'!

!MaHistoricalStat commentStamp: 'cmm 5/6/2008 13:00' prior: 0!
A MaHistoricalStat adds contextual information for easy rendering in user-interfaces.  It is used by MaStatHistory's.!

!MaHistoricalStat class methodsFor: 'as yet unclassified' stamp: 'cmm 6/26/2008 15:10'!
name: aString 
	^ self new
		name: aString ;
		yourself! !

!MaHistoricalStat methodsFor: 'summarize' stamp: 'cmm 6/27/2008 17:35'!
+ aMaStat 
	| answer myEndTime otherEndTime |
	answer := super + aMaStat.
	answer
		name: self name ;
		startTime: (answer startTime min: aMaStat startTime).
	myEndTime := self endTime ifNil: [ DateAndTime now ].
	otherEndTime := aMaStat endTime ifNil: [ DateAndTime now ].
	answer endTime: (myEndTime max: otherEndTime).
	^ answer! !

!MaHistoricalStat methodsFor: 'accessing' stamp: 'cmm 5/5/2008 23:55'!
endTime
	^endTime! !

!MaHistoricalStat methodsFor: 'accessing' stamp: 'cmm 5/5/2008 23:55'!
endTime: aDateAndTime
	endTime := aDateAndTime! !

!MaHistoricalStat methodsFor: 'initialize' stamp: 'cmm 5/6/2008 13:31'!
initialize
	super initialize.
	startTime := DateAndTime now! !

!MaHistoricalStat methodsFor: 'accessing' stamp: 'cmm 5/5/2008 23:55'!
name
	^name! !

!MaHistoricalStat methodsFor: 'accessing' stamp: 'cmm 5/5/2008 23:55'!
name: aString
	name := aString! !

!MaHistoricalStat methodsFor: 'accessing' stamp: 'cmm 5/5/2008 23:55'!
startTime
	^startTime! !

!MaHistoricalStat methodsFor: 'accessing' stamp: 'cmm 5/5/2008 23:55'!
startTime: aDateAndTime
	startTime := aDateAndTime! !

!MaStat class methodsFor: 'as yet unclassified' stamp: 'cmm 11/16/2004 10:51'!
example

	| stat |
	stat _ self new.
	#(1 2 5 8 9 15) do: [ :each | stat addValue: each ].
	stat! !

!MaStat methodsFor: 'summarize' stamp: 'cmm 6/26/2008 13:27'!
+ aMaStat 
	| n x c s |
	n := self min 
		ifNil: [ aMaStat min ]
		ifNotNil: 
			[ aMaStat min 
				ifNil: [ self min ]
				ifNotNil: [ self min min: aMaStat min ] ].
	x := self max 
		ifNil: [ aMaStat max ]
		ifNotNil: 
			[ aMaStat max 
				ifNil: [ self max ]
				ifNotNil: [ self max max: aMaStat max ] ].
	c := self count 
		ifNil: [ aMaStat count ]
		ifNotNil: 
			[ aMaStat count 
				ifNil: [ self count ]
				ifNotNil: [ self count + aMaStat count ] ].
	s := self sum 
		ifNil: [ aMaStat sum ]
		ifNotNil: 
			[ aMaStat sum 
				ifNil: [ self sum ]
				ifNotNil: [ self sum + aMaStat sum ] ].
	^ self class new 
		setLast: aMaStat last
		min: n
		max: x
		count: c
		sum: s! !

!MaStat methodsFor: 'accessing' stamp: 'cmm 5/7/2008 15:36'!
addValue: aMagnitude 
	"This is named addValue: instead of add: to be easier to find senders."
	last := aMagnitude.
	count := count + 1.
	min := min 
		ifNil: [ aMagnitude ]
		ifNotNil: [ min min: aMagnitude ].
	max := max 
		ifNil: [ aMagnitude ]
		ifNotNil: [ max max: aMagnitude ].
	sum := sum 
		ifNil: [ aMagnitude ]
		ifNotNil: [ sum + aMagnitude ].
	^ aMagnitude! !

!MaStat methodsFor: 'accessing' stamp: 'cmm 9/28/2007 12:31'!
count
	"The number of values."
	^count! !

!MaStat methodsFor: 'initialize' stamp: 'cmm 4/28/2008 22:56'!
initialize
	super initialize.
	count := last := 0! !

!MaStat methodsFor: 'accessing' stamp: 'cmm 9/28/2007 12:33'!
last
	"The last value recorded."
	^ last! !

!MaStat methodsFor: 'accessing' stamp: 'cmm 11/16/2004 21:50'!
last: aMagnitude

	last _ aMagnitude! !

!MaStat methodsFor: 'printing' stamp: 'cmm 11/16/2004 21:52'!
maPrintAbbreviatedOn: aStream

	self
		maPrintAttributes: #(min max sum count mean)
		on: aStream! !

!MaStat methodsFor: 'accessing' stamp: 'cmm 9/27/2007 17:23'!
max
	"The highest value."
	^max! !

!MaStat methodsFor: 'accessing' stamp: 'cmm 5/9/2008 13:59'!
mean
	"The average value."
	^ (sum isNil or: [ count isNil ]) 
		ifTrue: [ nil ]
		ifFalse: [ (sum / count) asFloat ]! !

!MaStat methodsFor: 'accessing' stamp: 'cmm 9/27/2007 17:23'!
min
	"The minimum value."
	^min! !

!MaStat methodsFor: 'initialize' stamp: 'cmm 4/28/2008 22:29'!
setLast: newLast min: newMin max: newMax count: newCount sum: newSum 
	last := newLast.
	min := newMin.
	max := newMax.
	count := newCount.
	sum := newSum! !

!MaStat methodsFor: 'accessing' stamp: 'cmm 9/27/2007 17:24'!
sum
	"The sum of all values."
	^ sum! !

MaObject subclass: #MaStatCollection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma Statistics-Domain'!

!MaStatCollection methodsFor: 'private-reporting' stamp: 'cmm 5/9/2008 14:28'!
countColumn
	^ (MaFixedWidthReportColumn dataAccessBlock: [ : aMaHistoricalStat | aMaHistoricalStat count ])
		headingText: 'Count' ;
		alignRight ;
		width: 8 ;
		repeatDuplicateValues: true ;
		useEllipsisOnTruncate: true ;
		yourself! !

!MaStatCollection methodsFor: 'private-reporting' stamp: 'cmm 5/9/2008 14:27'!
endTimeColumn
	^ (MaFixedWidthReportColumn dataAccessBlock: [ : aMaHistoricalStat | aMaHistoricalStat endTime ])
		headingText: 'End Time' ;
		alignLeft ;
		width: DateAndTime now printString size + 4 ;
		repeatDuplicateValues: true ;
		useEllipsisOnTruncate: false ;
		yourself! !

!MaStatCollection methodsFor: 'accessing' stamp: 'cmm 6/4/2008 10:34'!
ensureActive
	"My historys should only ever be started and stopped together, as a group."
	self isActive ifFalse: [ self transition ]! !

!MaStatCollection methodsFor: 'private-reporting' stamp: 'cmm 6/27/2008 17:40'!
historicalStatNameColumn
	^ (MaFixedWidthReportColumn dataAccessBlock: [ : eachHistory | eachHistory name ])
		headingText: 'Name' ;
		width: 40 ;
		useEllipsisOnTruncate: true ;
		repeatDuplicateValues: true ;
		yourself! !

!MaStatCollection methodsFor: 'private-reporting' stamp: 'cmm 6/27/2008 00:04'!
historicalStatRowSpec
	^ MaFixedWidthRowSpecification new
		addColumn: self endTimeColumn ;
		addColumn: self historicalStatNameColumn ;
		addColumn: self countColumn ;
		addColumn: self minColumn ;
		addColumn: self maxColumn ;
		addColumn: self sumColumn ;
		addColumn: self meanColumn ;
		addColumn: self lastColumn! !

!MaStatCollection methodsFor: 'accessing' stamp: 'cmm 5/7/2008 15:16'!
historys
	| answer |
	answer := OrderedCollection new.
	self historysDo: [ : each | answer add: each ].
	^ answer! !

!MaStatCollection methodsFor: 'subclass responsibility' stamp: 'cmm 5/7/2008 15:15'!
historysDo: aBlock 
	self subclassResponsibility! !

!MaStatCollection methodsFor: 'accessing' stamp: 'cmm 6/5/2008 10:28'!
isActive
	"My historys should only ever be started and stopped together, as a group."
	self historysDo: [ : anyOne | ^ anyOne isActive ].
	MaSoftwareError signal: 'never expected to reach here.'! !

!MaStatCollection methodsFor: 'private-reporting' stamp: 'cmm 5/9/2008 14:28'!
lastColumn
	^ (MaFixedWidthReportColumn dataAccessBlock: [ : aMaHistoricalStat | aMaHistoricalStat last ])
		headingText: 'last' ;
		alignRight ;
		width: 10 ;
		repeatDuplicateValues: true ;
		useEllipsisOnTruncate: true ;
		yourself! !

!MaStatCollection methodsFor: 'private-reporting' stamp: 'cmm 5/9/2008 14:28'!
maxColumn
	^ (MaFixedWidthReportColumn dataAccessBlock: [ : aMaHistoricalStat | aMaHistoricalStat max ])
		headingText: 'max' ;
		alignRight ;
		width: 10 ;
		repeatDuplicateValues: true ;
		useEllipsisOnTruncate: true ;
		yourself! !

!MaStatCollection methodsFor: 'private-reporting' stamp: 'cmm 5/9/2008 14:27'!
meanColumn
	^ (MaFixedWidthReportColumn dataAccessBlock: [ : aMaHistoricalStat | aMaHistoricalStat mean ])
		headingText: 'mean' ;
		alignRight ;
		width: 10 ;
		repeatDuplicateValues: true ;
		useEllipsisOnTruncate: false ;
		yourself! !

!MaStatCollection methodsFor: 'private-reporting' stamp: 'cmm 5/9/2008 14:27'!
minColumn
	^ (MaFixedWidthReportColumn dataAccessBlock: [ : aMaHistoricalStat | aMaHistoricalStat min ])
		headingText: 'min' ;
		alignRight ;
		width: 10 ;
		repeatDuplicateValues: true ;
		useEllipsisOnTruncate: true ;
		yourself! !

!MaStatCollection methodsFor: 'private-reporting' stamp: 'cmm 5/9/2008 14:27'!
nameColumn
	^ (MaFixedWidthReportColumn dataAccessBlock: [ : eachHistory | eachHistory name ])
		width: 75 ;
		useEllipsisOnTruncate: true ;
		yourself! !

!MaStatCollection methodsFor: 'reporting' stamp: 'cmm 5/9/2008 17:40'!
printReportTo: aWriteStream 
	"Print a report of all my stats to aWriteStream."
	aWriteStream
		maPrint: 'Note:  All times in milliseconds.' ;
		cr ;
		cr.
	self reportDefinition 
		write: self historys
		to: aWriteStream! !

!MaStatCollection methodsFor: 'initialize-release' stamp: 'cmm 5/7/2008 15:23'!
release
	"release each of my historys so their timer processes will be cleaned up."
	self historysDo: [ : each | each release ]! !

!MaStatCollection methodsFor: 'reporting' stamp: 'cmm 5/9/2008 17:40'!
report
	"Print a report of all my stats."
	^ String streamContents: [ : stream | self printReportTo: stream ]! !

!MaStatCollection methodsFor: 'private-reporting' stamp: 'cmm 6/27/2008 10:11'!
reportDefinition
	^ MaFixedWidthReport new
		for: MaStatHistory
			use: self statHistoryRowSpec ;
		for: MaStatHistory
			subReportOn: [ : eachStatHistory | eachStatHistory allStats ] ;
		for: MaStatHistory
			subReportOn: [ : eachStatHistory | Array with: eachStatHistory summary ] ;
		for: MaHistoricalStat
			use: self historicalStatRowSpec ;
		yourself! !

!MaStatCollection methodsFor: 'accessing' stamp: 'cmm 5/23/2008 22:57'!
snapshotInterval: anInterval 
	"An Interval that describes the period of the immediate past and the frequency to capture snapshots.  Specify the end-points of each temporal period.

For example, to specify history for the last 30 minutes, summarized every 5 minutes:

	5 to: 30 by: 5

To get minute-by-minute history for the last 24 hours:

	1 to: 24*60 by: 1

The first (receiver) Integer and the by: parameter will usually be the same."
	self historysDo: [ : each | each snapshotInterval: anInterval ]! !

!MaStatCollection methodsFor: 'private-reporting' stamp: 'cmm 5/9/2008 14:27'!
statHistoryRowSpec
	^ MaFixedWidthRowSpecification new
		addColumn: self nameColumn ;
		yourself! !

!MaStatCollection methodsFor: 'private-reporting' stamp: 'cmm 5/9/2008 14:27'!
sumColumn
	^ (MaFixedWidthReportColumn dataAccessBlock: [ : aMaHistoricalStat | aMaHistoricalStat sum ])
		headingText: 'sum' ;
		alignRight ;
		width: 10 ;
		repeatDuplicateValues: true ;
		useEllipsisOnTruncate: true ;
		yourself! !

!MaStatCollection methodsFor: 'accessing' stamp: 'cmm 6/3/2008 20:28'!
transition
	self historysDo: [ : each | each transition ]! !

MaObject subclass: #MaStatHistory
	instanceVariableNames: 'name startTime history snapshotInterval active timer guard'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma Statistics-Domain'!

!MaStatHistory commentStamp: 'cmm 5/2/2008 12:46' prior: 0!
A MaStatHistory tracks statistics for intervals of the recent past.  My #snapshotInterval describes the length of the recent past represented, as well as the duration of each snapshot within that recent past.

My #history represents the collection of MaStats captured during that recent past.!

!MaStatHistory class methodsFor: 'create' stamp: 'cmm 5/4/2008 23:19'!
name: aString 
	^ self new
		name: aString ;
		yourself! !

!MaStatHistory methodsFor: 'accessing' stamp: 'cmm 6/26/2008 15:13'!
active
	"Answer the stat that is still being tallied.  The interval (me) keeps track of the time elapsed, you just ask it for the #currentStat and add to that one."
	^ self guard critical: [ active ]! !

!MaStatHistory methodsFor: 'accessing' stamp: 'cmm 6/26/2008 15:13'!
addValue: aNumber 
	"Add aNumber in to the active stat (that for the most recent past) that is still being tallied."
	^ self guard critical: 
		[ self isActive 
			ifTrue: [ active addValue: aNumber ]
			ifFalse: [ aNumber ] ]! !

!MaStatHistory methodsFor: 'accessing' stamp: 'cmm 6/27/2008 10:11'!
allStats
	^ history , 
		(self isActive 
			ifTrue: [ Array with: active ]
			ifFalse: 
				[ #() ])! !

!MaStatHistory methodsFor: 'private' stamp: 'cmm 5/7/2008 16:47'!
archiveActive
	[ history size = snapshotInterval size ] whileTrue: [ history removeFirst ].
	active endTime: DateAndTime now.
	history addLast: active.
	active := nil! !

!MaStatHistory methodsFor: 'private' stamp: 'cmm 9/23/2008 14:25'!
guard
	^ guard ifNil: [ guard := Mutex new ]! !

!MaStatHistory methodsFor: 'accessing' stamp: 'cmm 4/28/2008 22:39'!
history
	^ history! !

!MaStatHistory methodsFor: 'initialize-release' stamp: 'cmm 9/23/2008 14:25'!
initialize
	super initialize.
	history := OrderedCollection new.
	guard := Mutex new.
	self snapshotInterval: (5 to: 30 by: 5)! !

!MaStatHistory methodsFor: 'testing' stamp: 'cmm 5/7/2008 17:09'!
isActive
	^ active notNil! !

!MaStatHistory methodsFor: 'accessing' stamp: 'cmm 6/26/2008 15:13'!
last
	^ self guard critical: 
		[ self isActive 
			ifTrue: [ active last ]
			ifFalse: [ 0 ] ]! !

!MaStatHistory methodsFor: 'accessing' stamp: 'cmm 5/2/2008 11:53'!
name
	^ name! !

!MaStatHistory methodsFor: 'accessing' stamp: 'cmm 5/2/2008 11:52'!
name: anObject
	name := anObject! !

!MaStatHistory methodsFor: 'private' stamp: 'cmm 6/3/2008 20:04'!
nextStat
	self
		resetTimer ;
		resetActive! !

!MaStatHistory methodsFor: 'initialize-release' stamp: 'cmm 6/26/2008 15:13'!
release
	"This should be called when no longer wishing to capture history so the timer processes will be cleaned up."
	self guard critical: 
		[ "#release should never blow up, even when it is malformed, therefore the nil check."
		timer ifNotNil: [ timer cancel ].
		self isActive ifTrue: [ self archiveActive ] ]! !

!MaStatHistory methodsFor: 'private' stamp: 'cmm 6/26/2008 15:11'!
resetActive
	self isActive ifTrue: [ self archiveActive ].
	active := MaHistoricalStat name: self name! !

!MaStatHistory methodsFor: 'private' stamp: 'cmm 6/5/2008 22:23'!
resetTimer
	"Begin a new timer that will transition after my snapshotDuration has elapsed.  Note use of MaTimer will produce an entry in the Process browser."
	"release just in case there was a prior timer."
	self release.
	timer := MaTimer new
		name: self name ;
		in: self snapshotDuration
			do: [ self transition ].
	startTime := DateAndTime now! !

!MaStatHistory methodsFor: 'accessing' stamp: 'cmm 5/2/2008 12:08'!
snapshotDuration
	"Answer the Duration of each MaStat."
	^ snapshotInterval increment minutes! !

!MaStatHistory methodsFor: 'accessing' stamp: 'cmm 5/9/2008 11:44'!
snapshotExplanation
	^ String streamContents: 
		[ : stream | 
		stream
			maPrint: 'Each ' ;
			maPrint: snapshotInterval increment ;
			maPrint: '-minute period for the last ' ;
			maPrint: snapshotInterval size ;
			maPrint: ' minutes' ]! !

!MaStatHistory methodsFor: 'accessing' stamp: 'cmm 5/2/2008 12:19'!
snapshotInterval
	"An Interval that describes the period of the immediate past and the frequency to capture snapshots.  Specify the end-points of each temporal period.

For example, to specify history for the last 30 minutes, summarized every 5 minutes:

	5 to: 30 by: 5

To get minute-by-minute history for the last 24 hours:

	1 to: 24*60 by: 1

The first (receiver) Integer and the by: parameter will usually be the same."
	^ snapshotInterval! !

!MaStatHistory methodsFor: 'accessing' stamp: 'cmm 6/26/2008 15:13'!
snapshotInterval: anInterval 
	"An Interval that describes the period of the immediate past and the frequency to capture snapshots.  Specify the end-points of each temporal period.

For example, to specify history for the last 30 minutes, summarized every 5 minutes:

	5 to: 30 by: 5

To get minute-by-minute history for the last 24 hours:

	1 to: 24*60 by: 1

The first (receiver) Integer and the by: parameter will usually be the same."
	self guard critical: 
		[ self release.
		snapshotInterval := anInterval ]! !

!MaStatHistory methodsFor: 'accessing' stamp: 'cmm 5/9/2008 11:23'!
startTime
	^ startTime! !

!MaStatHistory methodsFor: 'accessing' stamp: 'cmm 6/27/2008 17:37'!
summary
	| answer |
	answer := MaHistoricalStat name: 'Totals for ' , name , ':'.
	self guard critical: 
		[ history do: [ : each | answer := answer + each ].
		self isActive ifTrue: [ answer := answer + active copy ] ].
	^ answer! !

!MaStatHistory methodsFor: 'accessing' stamp: 'cmm 6/26/2008 15:13'!
transition
	^ self guard critical: 
		[ self isActive ifTrue: [ self archiveActive ].
		self nextStat ]! !

PackageInfo subclass: #MaStaticsPackageInfo
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma Statistics-Info'!

!MaStaticsPackageInfo methodsFor: 'as yet unclassified' stamp: 'cmm 5/9/2008 16:25'!
prerequisitePackages
	^ #('MaFixedWidthReport')! !
